"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringifyGitHubCommitPayload = exports.convertBodyToGitHubCommitPayload = exports.isGitHubCreateCommitEndpoint = void 0;
const logger = require("../../../log");
const errors_1 = require("./errors");
function isGitHubCreateCommitEndpoint(options) {
    // https://docs.github.com/en/rest/git/commits?apiVersion=2022-11-28#create-a-commit
    return options.method === 'POST' && options.url.includes('/git/commits');
}
exports.isGitHubCreateCommitEndpoint = isGitHubCreateCommitEndpoint;
function convertBodyToGitHubCommitPayload(body, options) {
    var _a;
    const bodyAsString = convertBodyToStringIfNeeded(body);
    try {
        const bodyAsJson = JSON.parse(bodyAsString);
        return {
            message: bodyAsJson.message,
            tree: bodyAsJson.tree,
            parents: bodyAsJson.parents,
            author: {
                name: bodyAsJson.author.name,
                email: bodyAsJson.author.email,
                date: convertToDate(bodyAsJson.author.date),
            },
            committer: {
                name: options.committerName,
                email: options.committerEmail,
                date: convertToDate(((_a = bodyAsJson.committer) === null || _a === void 0 ? void 0 : _a.date) || new Date()),
            },
        };
    }
    catch (error) {
        logger.error({ error }, 'Could not parse GitHub commit payload body');
        throw new errors_1.GitHubCommitParsingError('Could not parse GitHub commit payload body');
    }
}
exports.convertBodyToGitHubCommitPayload = convertBodyToGitHubCommitPayload;
const convertBodyToStringIfNeeded = (body) => {
    if (isUint8Array(body)) {
        return Buffer.from(body).toString();
    }
    else if (typeof body === 'string' || body instanceof String) {
        return body;
    }
    else {
        throw new Error('body must be string or Uint8Array');
    }
};
const isUint8Array = (data) => {
    return !!(data && data instanceof Uint8Array);
};
const convertToDate = (input) => {
    if (input && typeof input === 'string') {
        return new Date(input);
    }
    else if (input && input instanceof Date) {
        return input;
    }
    else {
        throw new Error('could not convert input to date');
    }
};
function stringifyGitHubCommitPayload(commit) {
    return `tree ${commit.tree}
parent ${commit.parents[0]}
author ${stringifyGitUser(commit.author)}
committer ${stringifyGitUser(commit.committer)}

${commit.message}`.trim();
}
exports.stringifyGitHubCommitPayload = stringifyGitHubCommitPayload;
const stringifyGitUser = (user) => {
    const timestamp = Math.floor(user.date.getTime() / 1000);
    return `${user.name} <${user.email}> ${timestamp} +0000`;
};
//# sourceMappingURL=commit.js.map